#!/bin/bash
#
# File: ksm-bzflagserver.sh
#
# The main program of the ksm-bzflagserver for a systemd system.
#
#    Copyright (C) 2009, 2017  Juergen Kaesmann (JK)

##############################################################################
# GPL-NOTICE

#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


##############################################################################
# EXIT CODES
# Standard

##############################################################################
# SOURCE FILES

. /usr/local/etc/main.conf				# config main
. $LLIBDIR/libfuncstd-sh				# functions general

##############################################################################
# DECLARATION OF CONSTANTS AND VARIABLES

scriptname=ksm-bzflagserver				# name of script
cnfdir='/usr/local/etc'					# config dir
mapfile="${cnfdir}/bzflag.conf"				# maps + options
s=''							# tmp variable
defmap=0						# numb of default map
acnf=1							# number of actual cnf
n=0							# number of lines
err=0							# error code

##############################################################################
# CHECK

if [ -f /etc/sysconfig/ksm-bzflagserver ]; then
	. /etc/sysconfig/ksm-bzflagserver
else
	echo "Could not find /etc/sysconfig/ksm-bzflagserver"
	echo "Please start the bzflagserver manually"
	exit 1
fi

if [ ! -f $mapfile ]; then
	echo "Could not find $mapfile"
	echo "Please start the bzflagserver manually"
	exit 1
fi

# Check for missing binaries (stale symlinks should not happen)
BZFLAGSERVER_BIN=/usr/bin/bzfs
test -x $BZFLAGSERVER_BIN || exit 5

# See /etc/sysconfig/bzflagserver for options to the bzfs
if [ "$BZFLAGSERVER_PUBLIC" == "yes" ] ; then
	PUBLIC="-public"
fi

if [ "$BZFLAGSERVER_NEWBIE" == "yes" ] ; then
	HANDICAP="-handicap"
fi

if [ "$BZFLAGSERVER_DISABLEBOTS" == "yes" ] ; then
	DISABLEBOTS="-disableBots"
fi

##############################################################################
# FUNCTIONS

#=============================================================================
function helptext ()
{
# Output helptext
# Uses cat, echo, logger
# Return codes:
# Standard

if [ "$debug" == "yes" ]; then				# DEBUG
	echo "DEBUG helptext: $# $@" >&2
	$debuglog helptext: $# $@
fi

get_defaults

cat <<EOT
NAME
$scriptname - Start the BZFlag Server

SYNOPSIS
$scriptname [options] [m n]

DESCRIPTION
Start the BZFlag Server.

OPTIONS
Gnu style options:

  -h|--help          = Output help, then exit.
  -V|--version       = Output version info, then exit.
  --start [n [-r]]   = Start the BZFlag Server.
  --stop             = Stop the BZFlag Server.
  --restart [n [-r]] = Restart the BZFlag Server.
  --status           = Show status of the BZFlag Server.
  --setconf [m]      = Set config to a preconfigured map file.
  -d|--debug         = Turn on DEBUG mode.
  -q|--quiet         = Be quiet.
  -v|--verbose       = Be verbose.
  -r                 = Type of game is RH (Rabbit Hunt).

ARGUMENTS
Give if needed:

  m = No. of preconfigured map file (actual No. = $acnf).
  n = No. of map (default map = $defmap).

EXIT STATUS
On program exit:

  Code 0 = OK
  Code 2 = Wrong parameter count
  Code 3 = Unspecified error
  Code 4 = Unknown option

  See output of 'showerrmsg'.

FILES
Used directories and files:

  Config Global = /etc/sysconfig/bzflag.conf
  Map Configs   = /usr/local/etc/bzflag.conf

SEE ALSO
Related manpages:

  showerrmsg(1), bzflag(), bzfs().

AUTHORS
Juergen Kaesmann <juergen@kaesmann-online.de>

MAPS
Following maps with default game style, options, number of the actual
map file and the default map are configured in $mapfile:

EOT
#EXAMPLES
#For instance:
#
#  Do it:
#  $scriptname

list_maps

return 0
}

#=============================================================================
function get_defaults ()
{
# get the default map file and actual bzflag map.
# uses cat, cut, wc.

# get number of lines in map file (1. line = random map,
# last line = number of default map)
n=`cat $mapfile | wc -l`

# get number of default map (out of actual conf)
defmap=`Get_Lines $mapfile $n $n | cut -f 1`

# get number of default config file (actual conf)
acnf=`Get_Lines $mapfile $n $n | cut -f 2`

return
}

#=============================================================================
function list_maps ()
{
# output a list of bzflag maps to STDOUT.
# uses cut, tr, wc.
# Return codes:
# Standard

local m s ss c i j l
m=''							# map
s=''							# style
ss=''							# space string
c=0							# column counter
i=0							# map counter
j=0							# loop counter
l=0							# len

let --n							# don't count defmap

i=0
while read; do
	let ++i						# map counter
	let ++c						# column counter
	m=$(echo "$REPLY" | tr -s '\t' | cut -f 1)
	s=$(echo "$REPLY" | tr -s '\t' | cut -f 2)

	l=$[30-${#m}]					# l = space len
	j=0
	ss=''						# ss = space string
	while [ "$j" -lt $l ]; do			# build space string
		let ++j
		ss="$ss "
	done

	l=${#s}						# l = len of style
	if [ "$l" -lt 3 ]; then				# unique len = 3
		s="$s "
	fi

	if [ "$i" -lt 10 ]; then			# len of number < 10 ?
		echo -n "  ${i} ${s} ${m}${ss}"
	else
		echo -n " ${i} ${s} ${m}${ss}"
	fi
	if [ "$c" -eq 2 ]; then				# after 2. map put EOL
		echo
		c=0
	fi
done<$mapfile

if [ "$c" -lt 2 ]; then					# need final EOL ?
	echo
fi

return
}

#=============================================================================
function set_map ()
{
# set the choosen bzflag map and the corresponding options.
# par1 = number of map. If empty then take values from default map.
# uses cut, tr, wc.
# Return codes:
# Standard

local map="$1" s='' ret=0

if [ -z "$map" ]; then
	map=$defmap
fi

if [ "$map" -ge 1 ] && [ "$map" -lt $n ]; then
	s=`Get_Lines $mapfile $map $map`
	BZFLAGSERVER_WORLD=`echo "$s" | tr -s '\t' | cut -f 1`
	BZFLAGSERVER_OPTS=`echo "$s" | tr -s '\t' | cut -f 3`
	ret=$?
fi

return
}

#=============================================================================
function check_conf_file ()
{
# Check a config file wether it exists.
# par1 = number of config file (1 - 99)
# If OK then push new config file name onto the stack (1 string).
# Return codes:
# Standard

local cfile='' p1="$1" ret=0

if [ "${#p1}" -eq 1 ]; then				# number less than 10
	p1="0$p1"
fi

cfile="${cnfdir}/bzflag.conf.${p1}"			# corr. cnf file name

if [ -e "$cfile" ]; then
	Push "$cfile"
else
	echo "${ErrMsg[5]} $cfile" >&2
	ret = 5
fi

return $ret
}

##############################################################################
# MAIN
case "$1" in
    --start)
	get_defaults
	set_map "$2"

	if [ "$3" == "-r" ]; then			# set game style RH
		BZFLAGSERVER_OPTS=$BZFLAGSERVER_OPTSR
	fi

	echo -n "Starting bzflagserver"

	if [ "$2" == "1" ]; then			# provided random map
		startproc -u nobody -g nobody $BZFLAGSERVER_BIN \
		$BZFLAGSERVER_OPTS $PUBLIC $HANDICAP $DISABLEBOTS \
		-p "$BZFLAGSERVER_PORT" \
		-srvmsg "_${BZFLAGSERVER_SRVMSG}_" \
		-admsg "_${BZFLAGSERVER_ADMSG}_"
	else						# use provided map
		startproc -u nobody -g nobody $BZFLAGSERVER_BIN \
		$BZFLAGSERVER_OPTS $PUBLIC $HANDICAP $DISABLEBOTS \
		-p "$BZFLAGSERVER_PORT" \
		-world "/usr/share/games/bzflag/maps/$BZFLAGSERVER_WORLD" \
		-srvmsg "_${BZFLAGSERVER_SRVMSG}_" \
		-admsg "_${BZFLAGSERVER_ADMSG}_"
	fi
	;;
    --stop)
	echo -n "Shutting down bzflagserver"
	## Stop daemon with killproc(8) and if this fails
	## set echo the echo return value.

	killproc -TERM $BZFLAGSERVER_BIN
	;;
    --restart)
	## Stop the service and regardless of whether it was
	## running or not, start it again.
	$0 stop
	$0 start $2 $3
	;;
    --status)
	echo -n "Checking for service bzflagserver: "
	## Check status with checkproc(8), if process is running
	## checkproc will return with exit status 0.

	# Return value is slightly different for the status command:
	# 0 - service running
	# 1 - service dead, but /var/run/  pid  file exists
	# 2 - service dead, but /var/lock/ lock file exists
	# 3 - service not running

	# NOTE: checkproc returns LSB compliant status values.
	checkproc $BZFLAGSERVER_BIN
	;;
    --setconf)
	check_conf_file "$2"
	err=$?
	if [ "$err" -eq 0 ]; then
		Pop s
		cp $s $mapfile
	fi
	;;
    -h|--help)
	helptext
	;;
esac

exit $err
